@@ -1,5 +1,5 @@
 /*
- *   Copyright 2017 by Ivan Cukic <ivan.cukic (at) kde.org>
+ *   Copyright 2017, 2018, 2019 by Ivan Cukic <ivan.cukic (at) kde.org>
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public License as
@@ -192,13 +192,13 @@ void PlasmaVaultService::registerVault(Vault *vault)
 {
     if (!vault->isValid()) {
         qWarning() << "Warning: Trying to register an invalid vault: "
-                   << vault->device();
+                   << vault->device().data();
         return;
     }
 
     if (d->knownVaults.contains(vault->device())) {
         qWarning() << "Warning: This one is already registered: "
-                   << vault->device();
+                   << vault->device().data();
         return;
     }
 
@@ -266,18 +266,18 @@ void PlasmaVaultService::onVaultStatusChanged(VaultInfo::Status status)
         // should be added or removed from the
         // inhibitors list
         const bool alreadyInhibiting =
-            devicesInhibittingNetworking.contains(vault->device());
+            devicesInhibittingNetworking.contains(vault->device().data());
 
         if (status == VaultInfo::Opened && !alreadyInhibiting) {
-            devicesInhibittingNetworking << vault->device();
+            auto deviceOpeningHandle = "{opening}" + vault->device().data();
+            devicesInhibittingNetworking.removeAll(deviceOpeningHandle);
+            devicesInhibittingNetworking << vault->device().data();
         }
 
         if (status != VaultInfo::Opened && alreadyInhibiting) {
-            devicesInhibittingNetworking.removeAll(vault->device());
+            devicesInhibittingNetworking.removeAll(vault->device().data());
         }
 
-        // qDebug() << "Vaults inhibitting networking: " << devicesInhibittingNetworking;
-
         // Now, let's handle the networking part
         if (!devicesInhibittingNetworking.isEmpty()) {
             NetworkManager::setNetworkingEnabled(false);
@@ -307,9 +307,18 @@ void PlasmaVaultService::onVaultMessageChanged(const QString &message)
 }
 
 
-void showPasswordMountDialog(Vault *vault, const std::function<void()> &function)
+template <typename OnAccepted, typename OnRejected>
+void showPasswordMountDialog(Vault *vault,
+                             OnAccepted onAccepted,
+                             OnRejected onRejected)
 {
-    auto dialog = new MountDialog(vault, function);
+    auto dialog = new MountDialog(vault);
+
+    QObject::connect(dialog, &QDialog::accepted,
+                     vault, onAccepted);
+    QObject::connect(dialog, &QDialog::rejected,
+                     vault, onRejected);
+
     dialog->open();
 }
 //^
@@ -321,10 +330,45 @@ void PlasmaVaultService::openVault(const QString &device)
     if (auto vault = d->vaultFor(device)) {
         if (vault->isOpened()) return;
 
+        if (vault->isOfflineOnly()) {
+            d->saveNetworkingState();
+
+            auto& devicesInhibittingNetworking = d->savedNetworkingState->devicesInhibittingNetworking;
+            auto deviceOpeningHandle = "{opening}" + vault->device().data();
+
+            // We need to check whether this vault
+            // should be added or removed from the
+            // inhibitors list
+            const bool alreadyInhibiting =
+                devicesInhibittingNetworking.contains(deviceOpeningHandle);
+
+            if (!alreadyInhibiting) {
+                devicesInhibittingNetworking << deviceOpeningHandle;
+            }
+
+            NetworkManager::setNetworkingEnabled(false);
+        }
+
+        auto stopInhibiting = [this, vault] {
+            if (d->savedNetworkingState) {
+                auto& devicesInhibittingNetworking = d->savedNetworkingState->devicesInhibittingNetworking;
+                auto deviceOpeningHandle = "{opening}" + vault->device().data();
+                devicesInhibittingNetworking.removeAll(deviceOpeningHandle);
+            }
+        };
+
         showPasswordMountDialog(vault,
-            [this, vault] {
-                emit vaultChanged(vault->info());
-            });
+                [this, vault, stopInhibiting] {
+                    emit vaultChanged(vault->info());
+                    stopInhibiting();
+                },
+                [this, vault, stopInhibiting] {
+                    stopInhibiting();
+                    if (vault->status() != VaultInfo::Opened) {
+                        d->restoreNetworkingState();
+                    }
+                }
+            );
     }
 }
 
@@ -367,13 +411,21 @@ void PlasmaVaultService::openVaultInFileManager(const QString &device)
 {
     if (auto vault = d->vaultFor(device)) {
         if (vault->isOpened()) {
-            new KRun(QUrl::fromLocalFile((QString)vault->mountPoint()), nullptr);
+            new KRun(QUrl::fromLocalFile((QString)vault->mountPoint().data()), nullptr);
 
         } else {
-            showPasswordMountDialog(vault, [this, vault] {
-                emit vaultChanged(vault->info());
-                new KRun(QUrl::fromLocalFile((QString)vault->mountPoint()), nullptr);
-            });
+            showPasswordMountDialog(vault,
+                [this, vault] {
+                    emit vaultChanged(vault->info());
+                    new KRun(QUrl::fromLocalFile((QString)vault->mountPoint().data()), nullptr);
+                },
+                [this, vault] {
+                    if (vault->status() != VaultInfo::Opened) {
+                        auto& devicesInhibittingNetworking = d->savedNetworkingState->devicesInhibittingNetworking;
+                        devicesInhibittingNetworking.removeAll(vault->device().data());
+                        d->restoreNetworkingState();
+                    }
+                });
         }
     }
 }
@@ -441,5 +493,15 @@ void PlasmaVaultService::deleteVault(const QString &device, const QString &name)
 }
 
 
+
+void PlasmaVaultService::updateStatus()
+{
+    for (const auto& device: d->knownVaults.keys()) {
+        auto vault = d->knownVaults[device];
+        vault->updateStatus();
+    }
+}
+
+
 #include "service.moc"
 
